﻿using UnityEngine;
using HarmonyLib;
using System.IO;
using System;

public class StatModifier : Mod
{
    public static string fileName
    {
        get
        {
            string date = new DateTime(SaveAndLoad.WorldToLoad.lastPlayedDateTicks).ToString(SaveAndLoad.dateTimeFormattingSaveFile);
            string text = Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.CurrentGameFileName, date);
            if (Directory.Exists(text))
                return Path.Combine(text, "StatModifier.json");
            return Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.CurrentGameFileName, date + SaveAndLoad.latestStringNameEnding, "StatModifier.json");
        }
    }
    public static JSONObject Config;
    static float[,] oldValues = null;
    private static float modiFValue = 1f;
    private static float modiWValue = 1f;
    public static float currentFoodModifier
    {
        get
        {
            return modiFValue;
        }
        set {
            modiFValue = value;
            Traverse fetcher = Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
            SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
            for (int i = 0;i < data.Length;i++)
            {
                data[i].nourishmentVariables.foodDecrementRateMultiplier = oldValues[i, 0] * modiFValue;
            }
        }

    }
    public static float currentWaterModifier
    {
        get
        {
            return modiWValue;
        }
        set
        {
            modiWValue = value;
            Traverse fetcher = Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
            SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
            for (int i = 0; i < data.Length; i++)
            {
                data[i].nourishmentVariables.thirstDecrementRateMultiplier = oldValues[i, 1] * modiWValue;
            }
        }

    }
    Harmony harmony;
    public void Start()
    {
        harmony = new Harmony("com.aidanamite.StatModifier");
        harmony.PatchAll();
        Traverse fetcher = Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
        SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
        oldValues = new float[data.Length, 2];
        for (int i = 0; i < data.Length; i++)
        {
            oldValues[i, 0] = data[i].nourishmentVariables.foodDecrementRateMultiplier;
            oldValues[i, 1] = data[i].nourishmentVariables.thirstDecrementRateMultiplier;
        }
        if (RAPI.GetLocalPlayer() != null)
            loadSaveJson();
        Log("Mod has been loaded!");
    }

    [ConsoleCommand(name: "statmodifier", docs: "Changes or returns current stat modifier")]
    public static string MyCommand(string[] args)
    {
        if (args.Length == 0)
            return "Hunger: " + currentFoodModifier.ToString() + "\nThirst: " + currentWaterModifier.ToString();
        if (args.Length > 1) 
            return "Too many arguments";
        try
        {
            float tmp = float.Parse(args[0]);
            currentFoodModifier = tmp;
            currentWaterModifier = tmp;
            return "Stat modifiers are now " + tmp.ToString();
        } catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    [ConsoleCommand(name: "statmodifier:food", docs: "Changes or returns current hunger stat modifier")]
    public static string MyCommand2(string[] args)
    {
        if (args.Length == 0)
            return currentFoodModifier.ToString();
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            currentFoodModifier = float.Parse(args[0]);
            return "Hunger stat modifier is now " + currentFoodModifier.ToString();
        }
        catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    [ConsoleCommand(name: "statmodifier:water", docs: "Changes or returns current thirst stat modifier")]
    public static string MyCommand3(string[] args)
    {
        if (args.Length == 0)
            return currentWaterModifier.ToString();
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            currentWaterModifier = float.Parse(args[0]);
            return "Thirst stat modifier is now " + currentWaterModifier.ToString();
        }
        catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    public override void WorldEvent_WorldLoaded()
    {
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll();
        Traverse fetcher = Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues");
        SO_GameModeValue[] data = fetcher.GetValue<SO_GameModeValue[]>();
        for (int i = 0; i < data.Length; i++)
        {
            data[i].nourishmentVariables.foodDecrementRateMultiplier = oldValues[i, 0];
            data[i].nourishmentVariables.thirstDecrementRateMultiplier = oldValues[i, 1];
        }
        Log("Mod has been unloaded!");
    }

    public static void loadSaveJson()
    {
        try
        {
            Config = new JSONObject(File.ReadAllText(fileName));
        } catch
        {
            Config = new JSONObject();
        }
        float currentModifier = 1;
        if (Config.HasField("Modifier"))
        {
            Config.GetField(ref currentModifier, "Modifier");
            Config.RemoveField("Modifier");
        }
        if (Config.HasField("Modifier_Food"))
        {
            currentFoodModifier = Config.GetField("Modifier_Food").n;
        }
        else
        {
            currentFoodModifier = currentModifier;
        }
        if (Config.HasField("Modifier_Water"))
        {
            currentWaterModifier = Config.GetField("Modifier_Water").n;
        }
        else
        {
            currentWaterModifier = currentModifier;
        }
    }

    public static void saveJson(string path)
    {
        if (Config.HasField("Modifier_Food"))
        {
            Config.SetField("Modifier_Food", StatModifier.currentFoodModifier);
        }
        else
        {
            Config.AddField("Modifier_Food", StatModifier.currentFoodModifier);
        }
        if (Config.HasField("Modifier_Water"))
        {
            Config.SetField("Modifier_Water", StatModifier.currentWaterModifier);
        }
        else
        {
            Config.AddField("Modifier_Water", StatModifier.currentWaterModifier);
        }
        try
        {
            if (!Config.IsNull)
                File.WriteAllText(path, Config.ToString());
        } catch (Exception err)
        {
            Log("An error occured while trying to save settings: " + err.Message);
        }
    }

    public static void Log(object message)
    {
        Debug.Log("[StatModifier]: " + message.ToString());
    }

    public void ExtraSettingsAPI_SettingsOpen()
    {
        ExtraSettingsAPI_SetSliderValue("Hunger Modifier", currentFoodModifier);
        ExtraSettingsAPI_SetSliderValue("Thirst Modifier", currentWaterModifier);
    }

    public void ExtraSettingsAPI_SettingsClose()
    {
        currentFoodModifier = ExtraSettingsAPI_GetSliderRealValue("Hunger Modifier");
        currentWaterModifier = ExtraSettingsAPI_GetSliderRealValue("Thirst Modifier");
    }
    static Traverse ExtraSettingsAPI_Traverse;
    static bool ExtraSettingsAPI_Loaded = false;
    public float ExtraSettingsAPI_GetSliderRealValue(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getSliderRealValue", new object[] { this, SettingName }).GetValue<float>();
        return 0;
    }
    public void ExtraSettingsAPI_SetSliderValue(string SettingName, float value)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("setSliderValue", new object[] { this, SettingName, value }).GetValue();
    }
}

[HarmonyPatch(typeof(SaveAndLoad), "Save")]
public class Patch_SaveGame
{
    static void Postfix(string filename)
    {
        string[] path = filename.Split(new char[] { '\\', '/' });
        filename = "";
        for (int i = 0; i < path.Length - 1; i++)
            filename += path[i] + "\\";
        if (filename.EndsWith(SaveAndLoad.latestStringNameEnding + "\\"))
            StatModifier.saveJson(filename + "StatModifier.json");
    }
}

[HarmonyPatch(typeof(LoadGameBox), "Button_LoadGame")]
public class Patch_LoadGame
{
    static void Postfix()
    {
        if (SaveAndLoad.WorldToLoad == null)
            return;
        StatModifier.loadSaveJson();
    }
}

[HarmonyPatch(typeof(NewGameBox), "Button_CreateNewGame")]
public class Patch_NewGame
{
    static void Postfix()
    {
        StatModifier.Config = new JSONObject();
        StatModifier.currentFoodModifier = 1;
        StatModifier.currentWaterModifier = 1;
    }
}